//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-imaging-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections.Generic;
using System.Diagnostics;
using GrapeCity.Documents.Drawing;
using GrapeCity.Documents.Imaging;
using DsImagingWeb.Demos.Common;
//
// This program creates an instance of the ShowLoClippingTiff class and uses it to generate an image file.
// The GrapeCity packages needed to build and run this sample are available on NuGet.org.
//
// Command line switches:
// /y, -y -- assume all defaults, overwrite existing files.
// /q, -q -- quiet operation.
// /n, -n -- generate repeatable 'random' data.
// /f path, -f path -- use 'name' as base file path/name.
//
var yes2all = false;
var quiet = false;
var norandom = false;
MemoryStream outMs = null;
TextWriter outTw = null;
try
{
    var fname = "ShowLoClippingTiff";
    for (int i = 0; i < args.Length; ++i)
    {
        switch (args[i].ToLowerInvariant())
        {
            case "/y":
            case "-y":
                yes2all = true;
                break;
            case "/q":
            case "-q":
                quiet = true;
                break;
            case "/n":
            case "-n":
                norandom = true;
                break;
            case "/f":
            case "-f":
                if (i + 1 < args.Length)
                {
                    if (Directory.Exists(args[++i]))
                        fname = Path.Combine(args[i], fname);
                    else
                        fname = args[i];
                }
                break;
        }
    }
    if (norandom)
        Util.NoRandom = true;
    if (!yes2all)
        quiet = false;
    if (quiet)
    {
        outMs = new MemoryStream();
        outTw = new StreamWriter(outMs);
        Console.SetOut(outTw);
    }

    var lk = Environment.GetEnvironmentVariable("DOCSOL_LKEY");
    if (!string.IsNullOrEmpty(lk))
        GcBitmap.SetLicenseKey(lk);

    // Find out which image creation method this sample supports, and what is its default image encoding:
    var sampleType = typeof(DsImagingWeb.Demos.ShowLoClippingTiff);
    var generateStream = sampleType.GetMethod("GenerateImageStream", new Type[] { typeof(string), typeof(Size), typeof(float), typeof(bool), typeof(string[]) });
    var generateImage = generateStream == null ? sampleType.GetMethod("GenerateImage", new Type[] { typeof(Size), typeof(float), typeof(bool), typeof(string[]) }) : null;
    var defaultMime = sampleType.GetProperty("DefaultMime");
    // Optional sample parameters:
    string[] sampleParams = null;
    if (sampleType.GetMethod("GetSampleParamsList", BindingFlags.Static | BindingFlags.Public)?.Invoke(null, null) is List<string[]> pars)
        sampleParams = pars[-1];

    var sample = new DsImagingWeb.Demos.ShowLoClippingTiff();
    // Adjust the target file name according to the image encoding:
    var imageMime = Util.MimeTypes.JPEG;
    if (defaultMime != null)
        imageMime = defaultMime.GetValue(sample) as string;
    var ext = "";
    switch (imageMime)
    {
        case Util.MimeTypes.JPEG:
            ext = ".jpeg";
            break;
        case Util.MimeTypes.PNG:
            ext = ".png";
            break;
        case Util.MimeTypes.TIFF:
            ext = ".tiff";
            break;
        case Util.MimeTypes.GIF:
            ext = ".gif";
            break;
        case Util.MimeTypes.BMP:
            ext = ".bmp";
            break;
        case Util.MimeTypes.ICO:
            ext = ".ico";
            break;
        case Util.MimeTypes.SVG:
            ext = ".svg";
            break;
        default:
            Debug.Assert(false);
            break;
    }
    fname = Path.ChangeExtension(fname, ext);

    if (!yes2all)
    {
        Console.WriteLine($"Press ENTER to create '{fname}' or enter an alternative file name:");
        var t = Console.ReadLine();
        if (!string.IsNullOrWhiteSpace(t))
            fname = t;
        fname = Path.GetFullPath(fname);
        if (File.Exists(fname))
        {
            Console.WriteLine($"'{fname}' exists.\nOVERWRITE (Y/n)?");
            var yesno = Console.ReadLine();
            if (yesno.ToLower() == "n")
            {
                Console.WriteLine("Aborted.");
                return;
            }
        }
    }

    // Generate the image using the method available for this sample:
    if (!quiet)
        Console.WriteLine($"Generating '{fname}'...");
    if (generateStream != null)
    {
        var stream = (Stream)generateStream.Invoke(sample, new object[] { imageMime, new Size(1024, 1024), 96, true, sampleParams });
        stream.Seek(0, SeekOrigin.Begin);
        using var fs = File.Create(fname);
        stream.CopyTo(fs);
    }
    else
    {
        var bmp = (GcBitmap)generateImage.Invoke(sample, new object[] { new Size(1024, 1024), 96, true, sampleParams });
        switch (imageMime)
        {
            case Util.MimeTypes.JPEG:
                bmp.SaveAsJpeg(fname, null, 70);
                break;
            case Util.MimeTypes.PNG:
                bmp.SaveAsPng(fname);
                break;
            case Util.MimeTypes.TIFF:
                bmp.SaveAsTiff(fname);
                break;
            case Util.MimeTypes.GIF:
                bmp.SaveAsGif(fname);
                break;
            case Util.MimeTypes.BMP:
                bmp.SaveAsBmp(fname);
                break;
            case Util.MimeTypes.ICO:
                bmp.SaveAsIco(fname, null, IcoFrameEncoding.Png);
                break;
            case Util.MimeTypes.SVG:
            default:
                Debug.Assert(false);
                break;
        }
    }
    if (!quiet)
        Console.WriteLine($"Created '{fname}' successfully.");

    // Uncomment to open the generated file:
    // ProcessStartInfo psi = new ProcessStartInfo() { FileName = fname, UseShellExecute = true };
    // Process.Start(psi);
}
catch (Exception ex)
{
    Console.Error.WriteLine($"Error: {ex.Message}");
    if (ex.InnerException != null)
        Console.Error.WriteLine($"InnerException: {ex.InnerException.Message}");
}
finally
{
    if (!yes2all)
    {
        Console.WriteLine("Press any key to exit...");
        Console.ReadKey();
    }
    if (quiet)
    {
        outTw.Dispose();
        outMs.Dispose();
    }
}
